c     Program to calculate the scattering from a sphere on or near a 
c     surface.  
c This program was written by Gorden Videen and modified by Dat Ngo
c It follows the derivation of Gorden Videen given in his dissertation
c and the articles
c Videen, Opt. Comm. 115, p. 1 (1995).
c Videen, JOSA A 8, p. 483 (1991); 9, p. 844 (1992).
c I can be reached at videen@atm.dal.ca

	implicit double precision (a-h,o-z)
	INCLUDE "surf.def"
	open(unit=4,file='surf.in', status='old')
	open(unit=8,file='surf.out')
c initial values read in from file "surf.in"

	read(4,*)wavel
	read(4,*)r
	read(4,*)n1,n2
	read(4,*)distance
	read(4,*)ns1,ns2
	read(4,*)thetao
	read(4,*)num

      ccc=dcmplx(0.0d0,1.0d0)
      thetai=0. 
      thetaf=360.
      mu=1.
      mus=1.

      pi=4.0*datan(1.d0)
      kmed=2.*pi*r/wavel 
      ksph=dcmplx(n1,n2)
      ksph=kmed*ksph
      kmir=dcmplx(ns1,ns2)
      kmir=kmir*kmed

c     nbg= # of iterations
      nbg=ksph+4.*ksph**.333 +2
c      print*,'Number of orders calculated: ',nbg
c      print*,'How many orders? '
c      read*,nbg_i
	nbg_i=nbg

      do 998 i_dist=1,1
        d = distance - 0.0010*(i_dist -1)
        kd = 4.*pi*d/wavel

	do 997 i_loop=1,1,1
	  nbg=nbg_i + i_loop

          nb4=nbg*4
c     Calculate Bessel Functions
      
      call newman(nbg,isize,besj,kmed)
      c=dcmplx(kmed,0.)
      call cbessel(nbg,isize,besh,c)
      do 11 i=-1,nbg
        besh(i)=besh(i)+ccc*besj(i)
        besj(i)=real(besh(i))
 11   continue

      call newman(nb4,isiz4,besjd,kd)
      c=dcmplx(kd,0.)
      call cbessel(nb4,isiz4,beshd,c)
      do 12 i=-1,nb4
        beshd(i)=beshd(i)+ccc*besjd(i)
 12   continue

      if (n1.gt.0.1) then
        call cbessel(nbg,isize,besjs,ksph)
      endif

c     Calculate Riccati-Bessel Functions and Derivatives needed

      besjs(-1)=ksph*besjs(-1)
      besj(-1)=kmed*besj(-1)
      besh(-1)=dcmplx(kmed)*besh(-1)

      do 100 i=0,nbg
        besjs(i)=ksph*besjs(i)
        besj(i)=kmed*besj(i)
        besh(i)=dcmplx(kmed)*besh(i)
        if (n1.gt.0.1) then
          dbesjs=besjs(i-1)-i*besjs(i)/ksph
        endif
        dbesj=besj(i-1)-i*besj(i)/kmed
        dbesh=besh(i-1)-i*besh(i)/dcmplx(kmed)
        if (n1.gt.0.1) then
        facte(i)=ksph*dbesjs*besj(i)-kmed*mu*dbesj*besjs(i)
        c=ksph*dbesjs*besh(i)-kmed*mu*dbesh*besjs(i)
        facte(i)=-facte(i)/c
        factf(i)=-kmed*mu*dbesjs*besj(i)+ksph*dbesj*besjs(i)
        c=ksph*dbesh*besjs(i)-kmed*mu*dbesjs*besh(i)
        factf(i)=-factf(i)/c
        else
          facte(i)=-besj(i)/besh(i)
          factf(i)=-dbesj/dbesh
        endif
 100  continue


c     Calculate scalar translation coefficients

      do 110 n=0,nbg*4
        cuplo(n)=beshd(n)*sqrt(2*n+1.)
        cuplo1(n)=-cuplo(n)
 110  continue
      do 115 n=0,nbg*2
        cuplom(0,n)=cuplo(n)
 115  continue

c     Now use equation C.15
      do 140 np=1,nbg*2
        do 120 n=0,nbg*4-np
          c=cdsqrt(dcmplx((2*n+1.)/(2*np-3.)))*(np-1.)
          c=cuplo1(n)*c
          cuplo1(n)=cuplo(n)
          cuplo(n)=c
 120    continue

        do 130 n=0,nbg*4-np
          c=-(n+1.)*sqrt((2*np-1.)/(2*n+3.))*cuplo1(n+1)
          c=c+n*cdsqrt(dcmplx((2*np-1.)/(2*n-1.)))*cuplo1(n-1)
          c=c+cuplo(n)
          cuplo(n)=c*sqrt((2*np+1.)/(2*n+1.))/np
          if (n.le.nbg*2) then
            cuplom(np,n)=cuplo(n) 
          endif
 130    continue
 140  continue

      theta=0
      state=1
      if (ns1.gt.0.1) then
        call reflect(reflect0,theta,kmed,kmir,mus,state)
      else
        reflect0 = -1.
      endif
      if (ns1.gt.0.1) then
        state=0
        theta=thetao*pi/180.
        call reflect(rper,theta,kmed,kmir,mus,state)
        state=1
        call reflect(rpar,theta,kmed,kmir,mus,state)
      else
        rpar = -1.
        rper = -1.
      endif

c Start iterations in m

      do 299, m=0,nbg
c	do 299 m=1,1
        print*,'m = ',m

c Calculate c coefficients

      if (m.gt.0) then
      do 155, np=m-1,2*nbg-m+1
         do 154, n=m-1,2*nbg-m+1
           cuplom1(np,n)=cuplom(np,n)
 154     continue
 155  continue
      do 160, np=m,2*nbg-m
        do 159, n =m,2*nbg-m
           c=sqrt((n-m+1.)*(n+m)*(2*n+1.))*cuplom1(np,n)
           cuplom(np,n)=c
           c=kd*sqrt((n-m+2.)*(n-m+1.)/(2*n+3.))
           cuplom(np,n)=cuplom(np,n)-c*cuplom1(np,n+1)
           c=kd*sqrt((n+m)*(n+m-1.)/(2*n-1.))
           cuplom(np,n)=cuplom(np,n)-c*cuplom1(np,n-1)
           c=sqrt((np-m+1.)*(np+m)*(2*n+1.))
           cuplom(np,n)=cuplom(np,n)/c
 159    continue
 160  continue
      endif 

c     Calculate Legendre Polynomials for incident angle

            ii=nbg+1
            x=dcos(pi*thetao/180.)
            mmm=m+1
            call nplgndr(mmm,ii,isize1,legpol1,x)
            mmm=m
            call nplgndr(mmm,ii,isize1,legpol2,x)
            if (m.gt.0) then
               mmm=m-1
               call nplgndr(mmm,ii,isize1,legpol,x)
            else
               do 165, jjj=-2,isize1
                  legpol(jjj)=-legpol1(jjj)
 165           continue
            endif

            do 180 n=m,nbg
                  tau=sqrt((n+m+1.)*(n-m))*legpol1(n)
                  pie=tau
                  tau=tau-sqrt((n+m)*(n-m+1.))*legpol(n)
                  tau=-tau*.5
                  if (dabs(x).gt.0.5) then
                    pie=pie+sqrt((n+m)*(n-m+1.))*legpol(n)
                    pie=pie*.5/x
                  else
                    pie=m*legpol2(n)/dsin(thetao*pi/180.)
                  endif
                  if (n.gt.0) then
                  aa(n)=-(ccc**n)/(n*(n+1))
                  ba(n)=aa(n)*pie
                  aa(n)=aa(n)*tau
                  endif
 180        continue

c load matrix for scattering coefficients

      numel=nbg-m+1
      numtot=numel+numel
      do 220 n=m,nbg
            i=n-m+1
            do 215 np=m,nbg
              ii=np-m+1
              iii=((-1)**(np+m))
c .........................................................
c . bigc and bigd are the vector translation coefficients .
c .........................................................
	      top=(n-m+1.)*(n+m+1.)
	      bottom=(2.*n+1.)*(2.*n+3.)
              bigc=cuplom(np,n+1)*sqrt(top/bottom)/(n+1.)
              bigd=0
              if (n.gt.0) then
		 top=(n-m)*(n+m)
		 bottom=(2.*n+1.)*(2.*n-1.)
                 bigc=bigc+cuplom(np,n-1)*sqrt(top/bottom)/n
                 bigd=-ccc*kd*m/n/(n+1.)*cuplom(np,n)
              endif
              bigc=cuplom(np,n)-kd*bigc

              c=-iii*facte(n)*reflect0*bigc
              matrixlu(i,ii)=c
              matrix(i,ii)=c
              c=iii*facte(n)*reflect0*bigd
              matrixlu(i,ii+numel)=c
              matrix(i,ii+numel)=c
              c=-iii*factf(n)*reflect0*bigd
              matrixlu(i+numel,ii)=c
              matrix(i+numel,ii)=c
              c=iii*factf(n)*reflect0*bigc
              matrixlu(i+numel,ii+numel)=c
              matrix(i+numel,ii+numel)=c
 215        continue
            matrix(i,i)=matrix(i,i)-1
            matrixlu(i,i)=matrix(i,i)
            matrix(i+numel,i+numel)=matrix(i+numel,i+numel)-1
            matrixlu(i+numel,i+numel)=matrix(i+numel,i+numel)
 220  continue

c Load up solution vectors
c For TE case
            do 226 n=m,nbg
              i=n-m+1
              c=cdexp(ccc*kd*x)
              c=c*((-1)**(n+m))
              b(i)=-facte(n)*aa(n)*(1.-rper*c)
              b(i+numel)=-factf(n)*ba(n)*(1.+rper*c)
              bd(i)=b(i)
              bd(i+numel)=b(i+numel)
 226        continue
            ii=isize1
            call ludcmp(matrixlu,numtot,ii,indx,dd)
            call lubksb(matrixlu,numtot,ii,indx,b)
            call mprove(matrix,matrixlu,numtot,ii,indx,bd,b)
  
c For TM case          
            do 228 n=m,nbg
              i=n-m+1
c .........................................................
c . Assigning values to scat coeff e() and f() for TE mode.
              e(m,n)=b(i)
              f(m,n)=b(i+numel)
c	e(m,n)=0.0
c	f(m,n)=0.0
c .........................................................
              c=cdexp(ccc*kd*x)
              c=c*((-1)**(n+m))
              b(i)=-ccc*facte(n)*ba(n)*(1.-rpar*c)
              b(i+numel)=-ccc*factf(n)*aa(n)*(1.+rpar*c)
              bd(i)=b(i)
              bd(i+numel)=b(i+numel)
 228        continue
            ii=isize1
            call lubksb(matrixlu,numtot,ii,indx,b)
            call mprove(matrix,matrixlu,numtot,ii,indx,bd,b)
c ..........................................................
c . Assigning values to scat coeff e() and f() for TM mode .
            do 229 n=m,nbg
              i=n-m+1
              e(isize1-m+1,isize1-n)=b(i)
              f(isize1-m+1,isize1-n)=b(i+numel)
c              e(isize1-m+1,isize1-n)=0.0
c              f(isize1-m+1,isize1-n)=0.0
 229        continue
c ..........................................................

 299  continue
c	print*,'DONE with solving the matrix!!'

c .....................................................
c .  Now to calculate amplitude coefficients and S11  .
c .....................................................
	do 399 i=1,num+1
            iquad2=2*num+1-(i-1)
            iquad3=2*num+1+(i-1)
            iquad4=4*num+1-(i-1)

            angle(i)=90.0*(i-1)/(num)
            angle(iquad2)=180.-angle(i)
            angle(iquad3)=180.+angle(i)
            angle(iquad4)=360.-angle(i)

            theta=angle(i)*pi/180.

c           Quadrants 2 and 3
            if (ns1.gt.0.1) then
               state=0
               call reflect(rper,theta,kmed,kmir,mus,state)
               state=1
               call reflect(rpar,theta,kmed,kmir,mus,state)
c           Quadrants 1 and 4
               state=2
               call reflect(tper,theta,kmed,kmir,mus,state)
               state=3
               call reflect(tpar,theta,kmed,kmir,mus,state)
            else
                rper = -1.
                rpar = -1.
                tper = 0.
                tpar = 0.
            endif

            sam1(i)=0.
            sam2(i)=0.
            sam1(iquad2)=0.
            sam2(iquad2)=0.
            sam1(iquad3)=0.
            sam2(iquad3)=0.
            sam1(iquad4)=0.
            sam2(iquad4)=0.

            x=dcos(theta)

            do 330 m=0,nbg
c                 Calculate Legendre polynomials
                  ii=nbg+1
                  mmm=m
                  call nplgndr(mmm,ii,isize1,legpol2,x)
                  mmm=m+1
                  call nplgndr(mmm,ii,isize1,legpol1,x)
                  if (m.eq.0) then
                        do 305, j=-2,isize1
                              legpol(j)=-legpol1(j)
 305                    continue
                  else
                        mmm=m-1
                        call nplgndr(mmm,ii,isize1,legpol,x)
                  endif

                  do 310 n=m,nbg      
c                       Calculate pie and tau for quad 1 and 4
                        tau=sqrt((n+m+1.)*(n-m))*legpol1(n)
                        pie=tau
                        tau=tau-sqrt((n+m)*(n-m+1.))*legpol(n)
                        tau=-tau*.5
                        if (dabs(x).gt.0.5) then
                          pie=pie+sqrt((n+m)*(n-m+1.))*legpol(n)
                          pie=pie*.5/x
                        else
                          pie=m*legpol2(n)/dabs(dsin(theta))
                        endif

c                       quadrants 1 and 4
                        b1=f(m,n)*pie+e(m,n)*tau
                        b2=e(isize1-m+1,isize1-n)*pie
                        b2=b2+f(isize1-m+1,isize1-n)*tau
                        bper=tper
                        bpar=tpar
                        ifact=(-ccc)**n
                        if (i.ne.num+1) then
c                       Quadrant 1
                           s1=2.*ifact*b1*bper
                           s2=2.*ifact*b2*bpar
                           if (m.lt.1) then
                              s1=s1/2.
                              s2=s2/2.
                           endif
                           sam1(i)=sam1(i)+s1
                           sam2(i)=sam2(i)-s2*ccc
c                       Quadrant 4
                           if (i.ne.1) then         
                              c=cdexp(ccc*m*pi)
                              sam1(iquad4)=sam1(iquad4)+s1*c
                              sam2(iquad4)=sam2(iquad4)-s2*ccc*c
                           endif
                        endif

c                       quadrants 2 and 3
                        pie=pie*(-1)**(n+m)
                        tau=tau*(-1)**(n+m-1)
                        b1=f(m,n)*pie+e(m,n)*tau
                        b3=f(m,n)*pie-e(m,n)*tau      
                        b2=e(isize1-m+1,isize1-n)*pie
                        b4=e(isize1-m+1,isize1-n)*pie
                        b2=b2+f(isize1-m+1,isize1-n)*tau
                        b4=-b4+f(isize1-m+1,isize1-n)*tau
                        
                        bper=(-1)**(n+m)
                        bper=bper*cdexp(ccc*kd*x)
                        bpar=rpar*bper
                        bper=rper*bper

c                       Quadrant 2
                        s1=2.*ifact*(b1+b3*bper)
                        s2=2.*ifact*(b2+b4*bpar)
                        if (m.lt.1) then
                           s1=s1/2.
                           s2=s2/2.
                        endif
                        sam1(iquad2)=sam1(iquad2)+s1
                        sam2(iquad2)=sam2(iquad2)-s2*ccc

c                       Quadrant 3
                        if (i.ne.1) then
                          c=cdexp(ccc*m*pi)
                          sam1(iquad3)=sam1(iquad3)+s1*c
                          sam2(iquad3)=sam2(iquad3)-s2*ccc*c
                        endif
 310              continue
 330        continue
 399  continue
 
c calculate matrix elements

      do 420, i=1,num*4
            c=sam1(i)*dconjg(sam1(i))
            cc=sam2(i)*dconjg(sam2(i))
            inten(1,i)=dreal(c)
            inten(2,i)=dreal(cc)
            mm(1,i)=dreal(c+cc)
            mm(1,i)=mm(1,i)/2.
            mm(2,i)=dreal(cc-c)
            mm(2,i)=mm(2,i)/2.
            mm(3,i)=dreal(sam1(i)*dconjg(sam2(i)))
            mm(4,i)=dimag(sam2(i)*dconjg(sam1(i)))
            if (ns1.gt.0.1) then
              mm(2,i)=mm(2,i)/mm(1,i)
              mm(3,i)=mm(3,i)/mm(1,i)
              mm(4,i)=mm(4,i)/mm(1,i)
            else
              if (i.gt.(num).and.i.lt.(3*num+2)) then
                mm(2,i)=mm(2,i)/mm(1,i)
                mm(3,i)=mm(3,i)/mm(1,i)
                mm(4,i)=mm(4,i)/mm(1,i)
              endif
            endif
c	print*,'i,angle(i),s11(i)=',i,angle(i),mm(1,i)
	    if((angle(i).gt.90.0).and.(angle(i).lt.270.0)) then
	   write(8,*)real(180.-angle(i)),inten(1,i),inten(2,i)	
c		write(8,*)real(180.-angle(i)),real(mm(1,i))
	    endif

 420  continue

c	print*,'d, nbg=',d,nbg
c	print*,'          s11(b)=',mm(1,2*num+1)
c	write(8,*)real(d),nbg,real(mm(1,2*num+1))

c     And that's all!
 997	continue
 998   continue

999   stop
      end

